----------------------------------------------------------------------------------------------------
-- Mouse Position Counter for VGA Display
-- by C.Meyer & c't magazin /2009
----------------------------------------------------------------------------------------------------
library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_unsigned.all;
  use ieee.numeric_std.all;

entity mouser is
  port (
    PCLK:	in  std_logic;
	HTICK: in  std_logic; -- these ticks are valid 2 PCLKs before pixel output
	VTICK: in  std_logic; -- valid with HTICK='1'
	MOUSESTYLE:	in std_logic_vector (7 downto 0);
	MOUSEX:	in std_logic_vector (7 downto 0);
	MOUSEY:	in std_logic_vector (7 downto 0);
	MOUSE:	out  std_logic;
	MOUSEBORDER:	out  std_logic
    );
end mouser;

architecture behave of mouser is
	signal hctr  : integer range 793 downto 0;
	signal vctr  : integer range 524 downto 0;
	signal pattern_line  : integer range 15 downto 0;
	signal pattern_pixel  : integer range 15 downto 0;
	signal cross_x: std_logic;
	signal cross_y: std_logic;
	signal valid_x: std_logic;
	signal valid_y: std_logic;
	signal mouse_pattern: std_logic_vector (15 downto 0);
	type rom_type is array (0 to 15)
        of std_logic_vector(15 downto 0);
   -- ROM definition of mouse arrow
	constant mouse_ROM: rom_type :=
	(  "0000000000000010",	-- 1. Zeile, spiegelverkehrt
	  "0000000000000110",	-- 2. Zeile, spiegelverkehrt usw.
	  "0000000000001110",
	  "0000000000011110",
	  "0000000000111110",
	  "0000000001111110",
	  "0000000011111110",
	  "0000000111111110",
	  "0000001111111110",
	  "0000000001111110", 
	  "0000000011100010",
	  "0000000011100000",
	  "0000000111000000",
	  "0000000111000000",
	  "0000000000000000",
	  "0000000000000000" -- 1. Bit und 16. Zeile muss leer sein
	);
	constant mouseborder_ROM: rom_type :=
	(  "0000000000000010",	-- 1. Zeile, spiegelverkehrt
	  "0000000000000110",	-- 2. Zeile, spiegelverkehrt usw.
	  "0000000000001010",
	  "0000000000010010",
	  "0000000000100010",
	  "0000000001000010",
	  "0000000010000010",
	  "0000000100000010",
	  "0000001111010010",
	  "0000000001011110", 
	  "0000000010100010",
	  "0000000010100000",
	  "0000000101000000",
	  "0000000111000000",
	  "0000000000000000",
	  "0000000000000000" -- 1. Bit und 16. Zeile muss leer sein
	);
   
	signal mouse_data: std_logic_vector(15 downto 0);
	signal mouse_bit: std_logic;
	signal mouseborder_data: std_logic_vector(15 downto 0);
	signal mouseborder_bit: std_logic;
	
begin
-- MOUSESTYLE: 00=aus, 01=Zeiger, 10=Haircross
with MOUSESTYLE(1 downto 0) select
		MOUSEBORDER <=	mouseborder_bit when "01", '0' when others;
with MOUSESTYLE(1 downto 0) select
		MOUSE <=	mouse_bit when "01", 
					(cross_x or cross_y) when "10",
					'0' when others;
	
--mouse_data <= mouse_ROM(pattern_line) when valid_y = '1' else x"0000";
--mouse_bit <= mouse_data(pattern_pixel) when valid_x = '1' else '0';
--mouseborder_data <= mouseborder_ROM(pattern_line) when valid_y = '1' else x"0000";
--mouseborder_bit <= mouseborder_data(pattern_pixel) when valid_x = '1' else '0';

mouse_data <= mouse_ROM(pattern_line);
mouseborder_data <= mouseborder_ROM(pattern_line);


mouse_count: process (PCLK)
begin
	if rising_edge(PCLK) then
		if valid_x='1' and valid_y='1' then
			mouse_bit <= mouse_data(pattern_pixel);
			mouseborder_bit <= mouseborder_data(pattern_pixel);
		else
			mouse_bit <= '0';
			mouseborder_bit <= '0';
		end if;
		
		if HTICK='1' then
			if VTICK='1' then
				vctr <= 0;
				pattern_line <= 15;
				valid_y <= '0';
			else
				vctr <= vctr+1;	
			end if;
			hctr <= 0;
			pattern_pixel <= 15;
			if vctr = MOUSEY & "00" then	-- auch hier geringere Auflsung
				cross_y <= '1';
				pattern_line <= 0;
				valid_y <= '1';
			else
				if pattern_line < 15 then
					pattern_line <= pattern_line+1;
					valid_y <= '1';
				end if;
				cross_y <= '0';	
			end if;
		else
			hctr<=hctr+1;
		end if;

		if hctr = MOUSEX & "00" then
			cross_x <= '1';
			pattern_pixel <= 0;
			valid_x <= '1';
		else
			if pattern_pixel < 15 then
				pattern_pixel <= pattern_pixel+1;
				valid_x <= '1';
			else
				valid_x <= '0';
			end if;
			cross_x <= '0';	
		end if;
	
	end if;
end process;

end behave;
