----------------------------------------------------------------------------------
-- Company:        c't magazin
-- Engineer:       Carsten Meyer, cm@ctmagazin.de
-- 
-- Create Date:    14:11:15 06/05/2010 
-- Design Name: 
-- Module Name:    tft_videomixer - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description:		Mixer for c't-BASIC video Interface
-- Dependencies: 
--
-- Revision: 
-- Revision 0.90 - Font size adjustable by constants
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity video_mixer is
    Port (	PCLK : in  STD_LOGIC;			-- pixel clock (about 10 MHz on DPF display 480 x 234), R75 on Haier Hi1018 PCB
			-- if VCOM_i unused, tie to '1'
			VCOM_i : in  STD_LOGIC:='0';	-- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
			TEXT_RED: in  STD_LOGIC;	
			TEXT_GREEN: in  STD_LOGIC;	
			TEXT_BLUE: in  STD_LOGIC;	
			TEXT_BIG: in  STD_LOGIC;	
			FRAMEVALID: in  STD_LOGIC;	
			
			GRAPHCOLOR: in  std_logic_vector(3 downto 0); 	-- Invert (3), R(2), G(1), B(0)
			FRAMECOLOR: in  std_logic_vector(3 downto 0); 	-- Invert (3), R(2), G(1), B(0)

			MOUSEARROW: in  STD_LOGIC;
			MOUSEBORDER: in  STD_LOGIC;
			GRAPHGRID: in  STD_LOGIC;
			GRAPHFRAME: in  STD_LOGIC;
			
			RED		: out std_logic_vector(1 downto 0);	-- color pixel output, to 3 transistor driver base C on Haier PCB
			GREEN	: out std_logic_vector(1 downto 0);
			BLUE	: out std_logic_vector(1 downto 0)
		   );
end video_mixer;

architecture behave of video_mixer is

signal mixed_red: std_logic_vector(2 downto 0) :=(others =>'0');
signal mixed_green: std_logic_vector(2 downto 0) :=(others =>'0');
signal mixed_blue: std_logic_vector(2 downto 0) :=(others =>'0');
signal graph_frame, btn_shadow,btn_shadow1,btn_shadow2,btn_video :  STD_LOGIC;

begin

-- Test:
--RED(1) 	<= TEXT_RED xnor VCOM_i; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
--GREEN(1) 	<= TEXT_GREEN xnor VCOM_i; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
--BLUE(1) 	<= TEXT_BLUE xnor VCOM_i; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
--RED(0) 	<= '0'; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
--GREEN(0) 	<= '0'; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
--BLUE(0) 	<= '0'; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB

--process (PCLK) begin	-- use this to delay and sync video out to PCLK clock
--	if rising_edge(PCLK) then

RED(1) 	<= mixed_red(2) xnor VCOM_i; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
RED(0) 	<= mixed_red(1) xnor VCOM_i;

GREEN(1) 	<= mixed_green(2) xnor VCOM_i; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
GREEN(0) 	<= mixed_green(1) xnor VCOM_i;

BLUE(1) 	<= mixed_blue(2) xnor VCOM_i; -- AC Component Vcom from display to invert RGB video, EC7 low side on Haier PCB
BLUE(0) 	<= mixed_blue(1) xnor VCOM_i;

mixed_red(2)	<= btn_video or TEXT_RED or (GRAPHCOLOR(2) and GRAPHFRAME and btn_shadow) when MOUSEARROW='0' else MOUSEBORDER;
mixed_green(2)	<= btn_video or TEXT_GREEN or (GRAPHCOLOR(1) and GRAPHFRAME and btn_shadow) when MOUSEARROW='0' else MOUSEBORDER;
mixed_blue(2)	<= btn_video or TEXT_BLUE or (GRAPHCOLOR(0) and GRAPHFRAME and btn_shadow) when MOUSEARROW='0' else MOUSEBORDER;

graph_frame		<= GRAPHFRAME and FRAMECOLOR(3);
mixed_red(1)	<= (((GRAPHGRID or FRAMECOLOR(2)) and graph_frame) and (not MOUSEARROW) and (not GRAPHCOLOR(2))) or MOUSEARROW or MOUSEBORDER;
mixed_green(1)	<= (((GRAPHGRID or FRAMECOLOR(1)) and graph_frame) and (not MOUSEARROW) and (not GRAPHCOLOR(1))) or MOUSEARROW or MOUSEBORDER;
mixed_blue(1)	<= (((GRAPHGRID or FRAMECOLOR(0)) and graph_frame) and (not MOUSEARROW) and (not GRAPHCOLOR(0))) or MOUSEARROW or MOUSEBORDER;

--	end if;
--end process;

btn_shadow1 <= TEXT_BIG;	-- unverzgert, Vorschatten
btn_shadow <= not(btn_shadow1 or btn_shadow2 or btn_video);


process (PCLK) begin	-- shadow generator for Button Overlay
	if rising_edge(PCLK) then
		btn_video <= TEXT_BIG;	-- delayed by 1 PCLK
		btn_shadow2 <= btn_video;	-- delayed by 2 PCLKs, Nachschatten
	end if;
end process;


end behave;
